//   Bluetooth Controlled NeoPixel Christmas Tree - By Diego Serodio Costa
// -------------------------------------------------------------------------

//Importing required libraries
#define BLYNK_USE_DIRECT_CONNECT
#include <Adafruit_NeoPixel.h>
#define BLYNK_PRINT DebugSerial
#include <BlynkSimpleSerialBLE.h>

// You could use a spare Hardware Serial on boards that have it (like Mega)
#include <SoftwareSerial.h>
SoftwareSerial DebugSerial(0, 1); // RX, TX

// Which pin on the Arduino is connected to the NeoPixels?
#define LED_PIN    2

// How many NeoPixels are attached to the Arduino?
#define LED_COUNT 180

// Declare our NeoPixel strip object:
Adafruit_NeoPixel strip(LED_COUNT, LED_PIN, NEO_GRB + NEO_KHZ800);

BlynkTimer timer; // Announcing the timer

// You should get Auth Token in the Blynk App.
// Go to the Project Settings (nut icon).
char auth[] = "AuthToken";

// Controls the current animation
int anim = 0;

// Colors components
int r = 0;
int g = 0;
int b = 0;

//Receiving commands from virtual pins
BLYNK_WRITE(V1)
{
  anim = param.asInt(); // assigning incoming value from pin V1 to a variable
}

BLYNK_WRITE(V2)
{
  anim = param.asInt(); // assigning incoming value from pin V2 to a variable
}

BLYNK_WRITE(V3)
{
  anim = 3; // assigning incoming value from pin V3 to a variable
  r = param[0].asInt();
  g = param[1].asInt();
  b = param[2].asInt();
}

BLYNK_WRITE(V4)
{
  strip.setBrightness(param.asInt());
  strip.show();
}

void setup()
{
  // Debug console
  DebugSerial.begin(9600);

  DebugSerial.println("Waiting for connections...");

  // Blynk will work through Serial
  // 9600 is for HC-06. For HC-05 default speed is 38400
  // Do not read or write this serial manually in your sketch
  Serial.begin(9600);
  Blynk.begin(Serial, auth);
  timer.setInterval(100L, toggleAnimation); //timer will run
  strip.begin();           // INITIALIZE NeoPixel strip object (REQUIRED)
  strip.show();            // Turn OFF all pixels ASAP
  strip.setBrightness(15); // Set BRIGHTNESS to about 1/5 (max = 255)
}

void loop()
{
  Blynk.run();
  timer.run();
}

void toggleAnimation(){
  switch(anim){
    case 0:
      strip.clear();
      strip.show();
      break;
    case 1:
      blinky();
      break;
    case 2:
      theaterChase();
      break;
    case 3:
      setColor();
      break;
  }
}

void blinky(){
  uint32_t color = strip.Color(random(0,256), random(0,256), random(0,256));
  strip.clear();
  for(int i=0; i<strip.numPixels(); i+=2) {
    strip.setPixelColor(i, color);
  }
  strip.show();
  delay(100);
  strip.clear();
  color = strip.Color(random(0,256), random(0,256), random(0,256));
  for(int i=1; i<strip.numPixels(); i+=2) {
    strip.setPixelColor(i, color);
  }
  strip.show();
  delay(100);
}

void theaterChase() {
  uint32_t color = strip.Color(random(0,256), random(0,256), random(0,256));
  for(int b=0; b<3; b++) { //  'b' counts from 0 to 2...
    strip.clear();         //   Set all pixels in RAM to 0 (off)
    // 'c' counts up from 'b' to end of strip in steps of 3...
    for(int c=b; c<strip.numPixels(); c += 3) {
      strip.setPixelColor(c, color); // Set pixel 'c' to value 'color'
    }
    strip.show(); // Update strip with new contents
    delay(100);  // Pause for a moment
  }
}

void setColor(){
  uint32_t color = strip.Color(r, g, b);
  for(int i=0; i<strip.numPixels(); i++) {
    strip.setPixelColor(i, color);
  }
  strip.show(); // Update strip with new contents
  delay(10);  // Pause for a moment
}
